"""Data models for the Eiswarnung API."""
from __future__ import annotations

from dataclasses import dataclass
from datetime import date, datetime
from enum import Enum
from typing import Any


class ForecastType(str, Enum):
    """Enumeration representing the Eiswarnung Forecast type."""

    NO_ICE = "No ice"
    POSSIBLY_ICE = "Possibly ice"
    ICE = "Ice"


@dataclass
class Forecast:
    """Object representing an Forecast response from Eiswarnung."""

    request_date: datetime | None
    status_id: int | None
    text: str | None
    city: str | None
    forecast_date: date | None

    @property
    def forecast_type(self) -> ForecastType:
        """Return API account_type information.

        Returns:
            The forecast type.
        """
        if self.status_id == 1:
            return ForecastType.ICE
        if self.status_id == 2:
            return ForecastType.POSSIBLY_ICE
        return ForecastType.NO_ICE

    @classmethod
    def from_response(cls, data: Any) -> Forecast:
        """Create an Forecast from a response.

        Args:
            data: The response data from the Eiswarnung API.

        Returns:
            A Forecast object.
        """
        data = data["result"]
        return Forecast(
            request_date=datetime.strptime(
                data.get("requestDate"), "%Y-%m-%d %H:%M:%S"
            ),
            status_id=data.get("forecastId"),
            text=data.get("forecastText").replace(".", ""),
            city=data.get("forecastCity"),
            forecast_date=datetime.strptime(
                data.get("forecastDate"), "%Y-%m-%d"
            ).date(),
        )


@dataclass
class Ratelimit:
    """Information about the current rate limit."""

    call_limit: int | None
    remaining_calls: int | None
    retry_after: int | None

    @classmethod
    def from_response(cls, data: dict[str, Any]) -> Ratelimit:
        """Create a Ratelimit object from the response data.

        Args:
            data: The response data from the Eiswarnung API.

        Returns:
            A Ratelimit object.
        """
        return cls(
            call_limit=data.get("callsDailyLimit"),
            remaining_calls=data.get("callsLeft"),
            retry_after=data.get("callsResetInSeconds"),
        )



"""Asynchronous client for the Eiswarnung API."""
from .eiswarnung import Eiswarnung
from .exceptions import (
    EiswarnungConnectionError,
    EiswarnungConnectionTimeoutError,
    EiswarnungError,
    EiswarnungRatelimitError,
    EiswarnungRequestError,
)
from .models import Forecast, Ratelimit

__all__ = [
    "Eiswarnung",
    "EiswarnungError",
    "EiswarnungConnectionTimeoutError",
    "EiswarnungConnectionError",
    "EiswarnungRatelimitError",
    "EiswarnungRequestError",
    "Forecast",
    "Ratelimit",
]



"""Asynchronous Python client for the Eiswarnung API."""
from __future__ import annotations

import asyncio
import socket
from collections.abc import Mapping
from dataclasses import dataclass
from importlib import metadata
from typing import Any

import aiohttp
import async_timeout
from aiohttp import hdrs
from yarl import URL

from .exceptions import (
    EiswarnungConnectionError,
    EiswarnungConnectionTimeoutError,
    EiswarnungError,
    EiswarnungRatelimitError,
    EiswarnungRequestError,
)
from .models import Forecast, Ratelimit


@dataclass
class Eiswarnung:
    """Main class for handling connections with the Eiswarnung API."""

    api_key: str
    latitude: float
    longitude: float

    request_timeout: float = 10.0
    session: aiohttp.client.ClientSession | None = None
    ratelimit: Ratelimit | None = None

    _close_session: bool = False

    async def _request(
        self,
        uri: str,
        *,
        method: str = hdrs.METH_GET,
        params: Mapping[str, str | float] | None = None,
    ) -> dict[str, Any]:
        """Handle a request to the Eiswarnung API.

        A generic method for sending/handling HTTP requests done against
        the Eiswarnung API.

        Args:
            uri: Request URI, without '/', for example, 'status'.
            method: HTTP method to use.
            params: Dictionary of params send to the Eiswarnung API.

        Returns:
            The response data from the Eiswarnung API.

        Raises:
            EiswarnungRequestError: There is something wrong with the
                variables used in the request.
            EiswarnungConnectionTimeoutError: A timeout occurred while
                communicating with the Eiswarnung API.
            EiswarnungConnectionError: An error occurred while communicating
                with the Eiswarnung API.
            EiswarnungError: Received an unexpected response from the
                Eiswarnung API.
            EiswarnungRatelimitError: The number of requests has exceeded
                the rate limit of the Eiswarnung API.
        """
        version = metadata.version(__package__)
        url = URL.build(scheme="https", host="api.eiswarnung.de", path="/").join(
            URL(uri)
        )

        headers = {
            "Accept": "application/json",
            "User-Agent": f"PythonEiswarnung/{version}",
        }

        if self.session is None:
            self.session = aiohttp.ClientSession()
            self._close_session = True

        try:
            async with async_timeout.timeout(self.request_timeout):
                response = await self.session.request(
                    method,
                    url,
                    params=params,
                    headers=headers,
                    ssl=True,
                )
                response.raise_for_status()
        except asyncio.TimeoutError as exception:
            raise EiswarnungConnectionTimeoutError(
                "Timeout occurred while connecting to Eiswarnung API"
            ) from exception
        except (aiohttp.ClientError, socket.gaierror) as exception:
            raise EiswarnungConnectionError(
                "Error occurred while communicating with Eiswarnung API"
            ) from exception

        content_type = response.headers.get("Content-Type", "")
        if "application/json" not in content_type:
            text = await response.text()
            raise EiswarnungError(
                "Unexpected response from the Eiswarnung API",
                {"Content-Type": content_type, "response": text},
            )

        data: dict[str, Any] = await response.json(content_type=None)

        if data["code"] == 200:
            self.ratelimit = Ratelimit.from_response(data)

        if data["code"] in [300, 400, 401]:
            raise EiswarnungRequestError(data)

        if data["code"] == 402:
            raise EiswarnungRatelimitError(data)

        return data

    async def forecast(self) -> Forecast:
        """Get forecast information from the Eiswarnung API.

        Returns:
            A Forecast object, with a ice warning forecast.
        """
        data = await self._request(
            "/",
            params={
                "key": self.api_key,
                "lat": self.latitude,
                "lng": self.longitude,
            },
        )
        return Forecast.from_response(data)

    async def close(self) -> None:
        """Close open client session."""
        if self.session and self._close_session:
            await self.session.close()

    async def __aenter__(self) -> Eiswarnung:
        """Async enter.

        Returns:
            The Eiswarnung object.
        """
        return self

    async def __aexit__(self, *_exc_info: Any) -> None:
        """Async exit.

        Args:
            _exc_info: Exec type.
        """
        await self.close()



"""Exceptions for Eiswarnung API."""
from typing import Any


class EiswarnungError(Exception):
    """General Eiswarnung exception."""


class EiswarnungConnectionError(EiswarnungError):
    """Eiswarnung connection exception."""


class EiswarnungRequestError(EiswarnungError):
    """Eiswarnung wrong request input variables."""

    def __init__(self, data: dict[str, Any]) -> None:
        """Init a solar request error.

        Args:
            data: The data that caused the error.

        https://www.eiswarnung.de/rest-api/
        """
        super().__init__(f'{data["message"]} (error {data["code"]})')
        self.code = data["code"]


class EiswarnungConnectionTimeoutError(EiswarnungError):
    """Eiswarnung connection timeout exception."""


class EiswarnungRatelimitError(EiswarnungError):
    """Eiswarnung ratelimit exception."""

    def __init__(self, data: dict[str, Any]) -> None:
        """Init a rate limit error.

        Args:
            data: The data that caused the error.
        """
        super().__init__(f'{data["message"]} (error {data["code"]})')
        self.code = data["code"]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['eiswarnung']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.0.0', 'yarl>=1.6.0']

setup_kwargs = {
    'name': 'eiswarnung',
    'version': '1.1.1',
    'description': 'Asynchronous Python client for the Eiswarnung API',
    'long_description': '## Python - Eiswarnung Client\n\n<!-- PROJECT SHIELDS -->\n[![GitHub Release][releases-shield]][releases]\n[![Python Versions][python-versions-shield]][pypi]\n![Project Stage][project-stage-shield]\n![Project Maintenance][maintenance-shield]\n[![License][license-shield]](LICENSE)\n\n[![GitHub Activity][commits-shield]][commits-url]\n[![Forks][forks-shield]][forks-url]\n[![Stargazers][stars-shield]][stars-url]\n[![Issues][issues-shield]][issues-url]\n[![GitHub Last Commit][last-commit-shield]][commits-url]\n\n[![Code Quality][code-quality-shield]][code-quality]\n[![Maintainability][maintainability-shield]][maintainability-url]\n[![Code Coverage][codecov-shield]][codecov-url]\n\n[![Build Status][build-shield]][build-url]\n[![Typing Status][typing-shield]][typing-url]\n\nAsynchronous Python client for the Eiswarnung API.\n\n## About\n\nA python package with which you can read data from [Eiswarnung API][eiswarnung]. This way you know whether it is necessary to scratch your car window the next day, because there is a layer of ice on it. This service comes from Germany but should work in other countries as well.\n\n## Installation\n\n```bash\npip install eiswarnung\n```\n\n## Usage\n\nFor a successful request, you must fill in the `latitude` and `longitude` data and have a valid `API key`, which you can request via [this website][request]. **Note**: there is a limit of 50 requests per day.\n\n```py\nimport asyncio\n\nfrom eiswarnung import Eiswarnung\n\nasync def main():\n    """Show example on getting data from the Eiswarnung API."""\n    async with Eiswarnung(\n        api_key="API_KEY",\n        latitude=49.17,\n        longitude=11.10,\n    ) as client:\n        forecast = await client.forecast()\n        print(forecast)\n        print(client.ratelimit)\n\nif __name__ == "__main__":\n    asyncio.run(main())\n```\n\n## Data\n\nYou can read the following data with this package:\n\n### Forecast\n\n- Request date (datetime)\n- Status ID (int)\n- Forecast Type - in English (str)\n- Forecast Text - in German (str)\n- Forecast City (str)\n- Forecast Date - the next day (datetime)\n\n### Ratelimit\n\n- Call Limit (int)\n- Remaining Calls (int)\n- Retry after (remaining seconds)\n\n## Contributing\n\nThis is an active open-source project. We are always open to people who want to\nuse the code or contribute to it.\n\nWe\'ve set up a separate document for our\n[contribution guidelines](CONTRIBUTING.md).\n\nThank you for being involved! :heart_eyes:\n\n## Setting up development environment\n\nThis Python project is fully managed using the [Poetry][poetry] dependencies\nmanager.\n\nYou need at least:\n\n- Python 3.9+\n- [Poetry][poetry-install]\n\nInstall all packages, including all development requirements:\n\n```bash\npoetry install\n```\n\nPoetry creates by default an virtual environment where it installs all\nnecessary pip packages, to enter or exit the venv run the following commands:\n\n```bash\npoetry shell\nexit\n```\n\nSetup the pre-commit check, you must run this inside the virtual environment:\n\n```bash\npre-commit install\n```\n\n*Now you\'re all set to get started!*\n\nAs this repository uses the [pre-commit][pre-commit] framework, all changes\nare linted and tested with each commit. You can run all checks and tests\nmanually, using the following command:\n\n```bash\npoetry run pre-commit run --all-files\n```\n\nTo run just the Python tests:\n\n```bash\npoetry run pytest\n```\n\n## License\n\nMIT License\n\nCopyright (c) 2021-2022 Klaas Schoute\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the "Software"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.\n\n<!-- PROJECT -->\n[eiswarnung]: https://www.eiswarnung.de\n[request]: https://www.eiswarnung.de/get-api\n\n<!-- MARKDOWN LINKS & IMAGES -->\n[build-shield]: https://github.com/klaasnicolaas/python-eiswarnung/actions/workflows/tests.yaml/badge.svg\n[build-url]: https://github.com/klaasnicolaas/python-eiswarnung/actions/workflows/tests.yaml\n[code-quality-shield]: https://img.shields.io/lgtm/grade/python/g/klaasnicolaas/python-eiswarnung.svg?logo=lgtm&logoWidth=18\n[code-quality]: https://lgtm.com/projects/g/klaasnicolaas/python-eiswarnung/context:python\n[commits-shield]: https://img.shields.io/github/commit-activity/y/klaasnicolaas/python-eiswarnung.svg\n[commits-url]: https://github.com/klaasnicolaas/python-eiswarnung/commits/main\n[codecov-shield]: https://codecov.io/gh/klaasnicolaas/python-eiswarnung/branch/main/graph/badge.svg?token=w0pbSPjFIZ\n[codecov-url]: https://codecov.io/gh/klaasnicolaas/python-eiswarnung\n[forks-shield]: https://img.shields.io/github/forks/klaasnicolaas/python-eiswarnung.svg\n[forks-url]: https://github.com/klaasnicolaas/python-eiswarnung/network/members\n[issues-shield]: https://img.shields.io/github/issues/klaasnicolaas/python-eiswarnung.svg\n[issues-url]: https://github.com/klaasnicolaas/python-eiswarnung/issues\n[license-shield]: https://img.shields.io/github/license/klaasnicolaas/python-eiswarnung.svg\n[last-commit-shield]: https://img.shields.io/github/last-commit/klaasnicolaas/python-eiswarnung.svg\n[maintenance-shield]: https://img.shields.io/maintenance/yes/2022.svg\n[maintainability-shield]: https://api.codeclimate.com/v1/badges/d1c7f7b99ac0225c2e18/maintainability\n[maintainability-url]: https://codeclimate.com/github/klaasnicolaas/python-eiswarnung/maintainability\n[project-stage-shield]: https://img.shields.io/badge/project%20stage-experimental-yellow.svg\n[pypi]: https://pypi.org/project/eiswarnung/\n[python-versions-shield]: https://img.shields.io/pypi/pyversions/eiswarnung\n[typing-shield]: https://github.com/klaasnicolaas/python-eiswarnung/actions/workflows/typing.yaml/badge.svg\n[typing-url]: https://github.com/klaasnicolaas/python-eiswarnung/actions/workflows/typing.yaml\n[releases-shield]: https://img.shields.io/github/release/klaasnicolaas/python-eiswarnung.svg\n[releases]: https://github.com/klaasnicolaas/python-eiswarnung/releases\n[stars-shield]: https://img.shields.io/github/stars/klaasnicolaas/python-eiswarnung.svg\n[stars-url]: https://github.com/klaasnicolaas/python-eiswarnung/stargazers\n\n<!-- Development -->\n[poetry-install]: https://python-poetry.org/docs/#installation\n[poetry]: https://python-poetry.org\n[pre-commit]: https://pre-commit.com\n',
    'author': 'Klaas Schoute',
    'author_email': 'hello@student-techlife.com',
    'maintainer': 'Klaas Schoute',
    'maintainer_email': 'hello@student-techlife.com',
    'url': 'https://github.com/klaasnicolaas/python-eiswarnung',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
